﻿using System.ComponentModel;

namespace TestProject
{
    /// <summary>
    /// 기본 객체
    /// </summary>
    public class BaseObject : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 - PropertyChanged

        /// <summary>
        /// 속성 변경시
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 명칭
        /// </summary>
        private string name;

        /// <summary>
        /// 실행자
        /// </summary>
        private string executor;

        /// <summary>
        /// 소유자 진행 객체
        /// </summary>
        private ProgressingObject ownerProgressingObject;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 명칭 - Name

        /// <summary>
        /// 명칭
        /// </summary>
        public string Name
        {
            get
            { 
                return this.name;
            }
            set
            {
                if(this.name == value)
                {
                    return;
                }

                this.name = value;

                OnPropertyChanged("Name");
            }
        }

        #endregion
        #region 실행자 - Executor

        /// <summary>
        /// 실행자
        /// </summary>
        public string Executor
        {
            get
            {
                return this.executor;
            }
            set
            {
                if(ReferenceEquals(this.executor, value))
                {
                    return;
                }

                this.executor = value;

                OnPropertyChanged("Executor");
            }
        }

        #endregion
        #region 소유자 진행 객체 - OwnerProgressingObject

        /// <summary>
        /// 소유자 진행 객체
        /// </summary>
        public ProgressingObject OwnerProgressingObject
        {
            get
            {
                return this.ownerProgressingObject;
            }
            set
            {
                if(ReferenceEquals(this.ownerProgressingObject, value))
                {
                    return;
                }

                this.ownerProgressingObject = value;

                OnPropertyChanged("OwnerProgressingObject");
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 문자열 구하기 - ToString()

        /// <summary>
        /// 문자열 구하기
        /// </summary>
        /// <returns>문자열</returns>
        public override string ToString()
        {
            return this.name;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 속성 변경시 처리하기 - OnPropertyChanged(propertyName)

        /// <summary>
        /// 속성 변경시 처리하기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}